# The Q-irreducible representations of A_n are as follows:
# The Q-irreducible reps of S_n are the same as the C-irreducible reps 
# and are determined by Young diagrams (partitions of n).  The 
# restriction of any irreducible rep of S_n to A_n is Q-irreducible.
# Two representations of S_n give the same rep of A_n iff the young
# diagrams are conjugate.  The induced rep of A_n is irreducible over 
# C if and only if the young diagram is not equal to its conjugate. 
# If the diagram _is_ symmetric, then it factors over a quadratic 
# extension of Q.  

Read("RepsOfS_n/RepsOfS_5");
RepresentationsOfA5 := function()
    local reps, dims, mult, rank, G;
    G := SymmetricGroup(5);
    reps := [RepOfS_5_5(G), RepOfS_5_14(G), RepOfS_5_23(G),
             RepOfS_5_113(G)];
    mult := [1, 4, 5, 3];
    
    #Check
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    rank := Sum(List([1..Length(reps)], 
                        i -> mult[i]*dims[i]));
    Assert(-1,2*rank = Order(G));
    
    return [mult, reps];
end;
    
Read("RepsOfS_n/RepsOfS_6");
RepresentationsOfA6 := function()
    local reps, dims, mult, rank, G;
    G := SymmetricGroup(6);
    reps := [RepOfS_6_6(G), RepOfS_6_15(G), RepOfS_6_24(G), RepOfS_6_114(G),
             RepOfS_6_33(G), RepOfS_6_123(G)];
    mult := [1, 5, 9, 10, 5, 8];
    
    #Check
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    rank := Sum(List([1..Length(reps)], 
                        i -> mult[i]*dims[i]));
    Assert(-1,2*rank = Order(G));
    
    return [mult, reps];
end;

Read("RepsOfS_n/RepsOfS_7");
RepresentationsOfA7 := function()
    local reps, dims, mult, rank, G;
    G := SymmetricGroup(7);
    reps := [RepOfS_7_7(G), RepOfS_7_16(G), RepOfS_7_25(G), 
             RepOfS_7_115(G), RepOfS_7_34(G), RepOfS_7_124(G),
             RepOfS_7_1114(G), RepOfS_7_133(G)]; 
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    
    mult :=  StructuralCopy(dims);
    mult[7] := mult[7]/2;
    
    #Check
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    rank := Sum(List([1..Length(reps)], 
                        i -> mult[i]*dims[i]));
    Assert(-1,2*rank = Order(G));
    
    return [mult, reps];
end;

Read("RepsOfS_n/RepsOfS_8");
RepresentationsOfA8 := function()
    local reps, dims, mult, rank, G;
    G := SymmetricGroup(8);
    reps := [RepOfS_8_8(G), RepOfS_8_17(G), RepOfS_8_26(G), 
             RepOfS_8_116(G), RepOfS_8_35(G), RepOfS_8_125(G),
             RepOfS_8_44(G), RepOfS_8_134(G),
             RepOfS_8_224(G), RepOfS_8_1124(G),RepOfS_8_11114(G),
             RepOfS_8_233(G)];
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    
    mult :=  StructuralCopy(dims);
    mult[10] := mult[10]/2;
    mult[12] := mult[12]/2;
    
    #Check
    dims := List(reps, r -> DimensionsMat(Image(r, GeneratorsOfGroup(G)[1]))[1]);
    rank := Sum(List([1..Length(reps)], 
                        i -> mult[i]*dims[i]));
    Assert(-1,2*rank = Order(G));
    
    return [mult, reps];
end;

RepresentationsOfAlternatingGroups := [0, 0, 0, 0, RepresentationsOfA5, 
                                       RepresentationsOfA6, RepresentationsOfA7, RepresentationsOfA8];


ComputeDataOnAltRepresentations := function(manifolds, n)
    local ident, Q, M, G, quo_rec, dims, reps, i, f,
          exp_rank, temp, data, rank,ranks, mult;
    Q := AlternatingGroup(n);
    ident := [Order(Q), 1];
    temp := RepresentationsOfAlternatingGroups[n]();
    mult := temp[1];
    reps := temp[2];
    data := [];
    for M in manifolds do
        for quo_rec in M.Quotients do
            if quo_rec.Type = "simple" and quo_rec.Ident = ident then
                # Compute locally
                ranks := List(reps, rep -> 
                              ComputeTwistedHomologyAlt(M, Q, quo_rec, rep));
                # compute globally
                #temp := GetSimpleQuotient(M, quo_rec);
                #G := temp[1];
                #f := temp[3];
                #rank := AbelianRankOfSubgroup(G, Kernel(f));
                #Add(data, [rank, ranks]);
                # Check that the ranks agree
                exp_rank := Sum(List([1..Length(reps)], 
                                    i -> mult[i]*ranks[i]));
                #Assert(-1, exp_rank = rank);
                Add(data, [exp_rank, ranks]);
            fi;
        od;
    od;
    return data;
end;

ExtractAltData := function(data)
    local dat, i, extracted;
    
    extracted := [[]];
    for dat in data[1][2] do
        Add(extracted, []);
    od;
        
    for dat in data do
        Add(extracted[1], dat[1]);
        for i in [1..Length(dat[2])] do
            Add(extracted[i+1], dat[2][i]);
        od;
    od;
    return extracted;
end;

tally := function(L)
    return List(AsSet(L), i -> [i,Number(L, j -> j = i)]);
end;
