FindChars := function( chartable, charlist)
# Find a minimal subset of charlist, greedy with the fewest
# characters at the end, that contains all of Irr(chartable)

    local nirr, sc, msp, indexset, ii,kk, jj, irr, irr2do;
    irr := Irr(chartable);
    indexset := [];
    nirr :=  Size(irr);
    irr2do := [1..nirr];
    sc := Size(charlist);
    msp := MatScalarProducts(charlist,Irr(chartable));
    if 0 in MatScalarProducts( [Sum(charlist)], irr) then
    	Print("Not enough characters to hit all irreducibles!");
	return fail;
    fi;
    while Size(irr2do) > 0 do
	jj := Maximum( List(irr2do, ii ->First( [1..sc], kk -> msp[ii][kk] > 0)));
	Add(indexset, jj);
	irr2do := Filtered( irr2do, ii -> msp[ii][jj] = 0);
	#Print(Size(irr2do), "...\n");
    od;
    return indexset;
end;

ChooseSubgroups := function( grp )
local ct, sgs, pcs, nsg, indices, iii, HH, JJ, curhits, GG;
   GG := ShallowCopy(grp);
    ct := CharacterTable(GG);
    sgs := List(ConjugacyClassesSubgroups(GG), Representative);
    nsg := Size(sgs);
    Print( nsg , " subgroups of index\n");
    pcs := [];
    Sort( sgs, function(HH, JJ) return  Order(JJ) <= Order(HH); end);
    Print( List(sgs, HH ->Index(GG, HH) ) , "\n");
    for iii in [1..nsg] do
    	Add( pcs, PermutationCharacter( GG, sgs[iii]) );
	curhits := MatScalarProducts( ct, Irr(ct), [Sum(pcs)] );
	Print("Adding index ", Index(GG, sgs[iii]), ", hits: ",curhits, "\n");
	if not 0 in curhits[1] then break; fi;
    od;
    Print("\n");
    indices := FindChars( ct, pcs );
    Print("Reduced to indices ", List(sgs{indices},  HH -> Index(GG,HH) ),"\n");
    return sgs{ indices };
end;


# For large alternating and symmetric groups, it's just impossible to
# compute all of the subgroups.  Instead we have the following
# functions for producing at least some subgroups.

ProductOfGroups := function(partition, GroupTypeGroup)
    local gens, curr, next,k,  G;
    gens := [()];
    curr := 1;
    for k in partition do
        if k > 1 then
            next := curr + k;
            G := GroupTypeGroup([curr..(next - 1)]);
            Append(gens, GeneratorsOfGroup(G));
            curr := next;
        fi;
    od;
    return Group(gens);
end;
                 
ApproxLowIndexSubgroups := function(n, GroupTypeGroup)
    local G, sgs, partitions, part, m;
    G := GroupTypeGroup(n);
    sgs := [];
    # Transitive groups from library
    for m in [3..n] do 
        Append(sgs, List([1..NrTransitiveGroups(m)], i -> TransitiveGroup(m,i)));
    od;
    sgs := Filtered(sgs, H -> IsSubgroup(G, H));
    # Products of Alternating/Symmetric Groups
    Append(sgs, List(Partitions(n), p -> ProductOfGroups(p, GroupTypeGroup)));
    # Stablizers of conjucagy classes.  
    Append(sgs, List(ConjugacyClasses(G), Stabilizer));
    
    # pick the right groups:
    
    sgs := Unique(sgs);
    Sort(sgs, function(H,K) return  Order(K) <= Order(H); end);
    
    return sgs;
    
end;

# NChooseSubgroups is the same as ChooseSubgroups but returns all the
# subgroups with index <= the index of the smallest subgroup in
# (indices).  Note that it has an artificial parameter "maxindex"
# when dealing with large symmetric and alternating groups.  

NChooseSubgroups := function( grp )
local ct, sgs, pcs, nsg, indices, iii, HH, JJ, curhits, GG, maxindex, n;
   GG := ShallowCopy(grp);
   ct := CharacterTable(GG);
   
   if IsAbelian(grp) then
       return List(ConjugacyClassesSubgroups(grp), Representative);
   fi;
   
   
   maxindex := 5000;
   if IsNaturalAlternatingGroup(grp) and NrMovedPoints(grp) >= 10 then
       n := NrMovedPoints(grp);
       sgs := Filtered(ApproxLowIndexSubgroups(n, AlternatingGroup), H -> Index(grp, H) < maxindex);
       return sgs;
   else if IsNaturalSymmetricGroup(grp) and NrMovedPoints(grp) >= 10 then
       n := NrMovedPoints(grp);
       sgs := Filtered(ApproxLowIndexSubgroups(n, SymmetricGroup), H -> Index(grp, H) < maxindex);
       return sgs;
   else
       sgs := List(ConjugacyClassesSubgroups(GG), Representative);
   fi; fi;
   nsg := Size(sgs);
   Print( "     ", nsg , " subgroups of index\n");
   pcs := [];
   Sort( sgs, function(HH, JJ) return  Order(JJ) <= Order(HH); end);
   Print( "       ", List(sgs, HH ->Index(GG, HH) ) , "\n");
   #Print("CHEATING\n");
   #return Filtered(sgs, HH -> Index(GG, HH) < 1000);
   for iii in [1..nsg] do
       Add( pcs, PermutationCharacter( GG, sgs[iii]) );
       curhits := MatScalarProducts( ct, Irr(ct), [Sum(pcs)] );
       #Print("Adding index ", Index(GG, sgs[iii]), ", hits: ",curhits, "\n");
       if not 0 in curhits[1] then break; fi;
   od;
   #Print("\n");
   indices := FindChars( ct, pcs );
   Print("         Reduced to indices ", List(sgs{indices},  HH -> Index(GG,HH) ),"\n");
   return sgs{ [1..Maximum(indices)] };
end;
