#Initialize Perfect Gp Lib

PerfectGroup(60);

SizesOfSimpleGroups := function()
   local L;
   L :=  List( PERFRec.sizeNumberSimpleGroup, x -> x[1]);
   return L;
end;

NNNSizesOfSimpleGroups := SizesOfSimpleGroups();


IdentifiersOfSimpleGroups := function()
    local L, pair;
    L := [];
    for pair in Collected(NNNSizesOfSimpleGroups) do
       Append(L,  List([1..pair[2]], x -> [pair[1], x]));
    od;
    return L;
end;

IdentifiersOfNonL2Groups := function()
   local PSLsizes, helper;
   helper := function(p)
     if (p mod 2) = 0 then 
         return p^3 - p;
     else 
         return (p^3 - p)/2;
     fi; end;
   PSLsizes := List(Filtered([5..125], IsPrimePowerInt), helper);
   return Filtered(IdentifiersOfSimpleGroups(), x -> not x[1] in PSLsizes);
end;

NNNIdentifiersOfSimpleGroups := IdentifiersOfSimpleGroups();

SimpleGroupIndex := function(group)
   if IsInt(group) then 
      return Position(NNNSizesOfSimpleGroups, group);
   fi;
   if IsGroup(group) then
      return Position(NNNSizesOfSimpleGroups, Order(group));
   fi;
end;

SimpleGroup := function(indent) 
   local L, size;
   size := indent[1];
   L := Filtered(PERFRec.sizeNumberSimpleGroup, x -> x[1] = size);
   return PerfectGroup(IsPermGroup, L[indent[2]]);
end;


NNNSimpleGroupNames :=
[ [ 60, "A5" ], [ 168, "L3(2)" ], [ 360, "A6" ], [ 504, "L2(8)" ], 
  [ 660, "L2(11)" ], [ 1092, "L2(13)" ], [ 2448, "L2(17)" ], [ 2520, "A7" ], 
  [ 3420, "L2(19)" ], [ 4080, "L2(16)" ], [ 5616, "L3(3)" ], 
  [ 6048, "U3(3)" ], [ 6072, "L2(23)" ], [ 7800, "L2(25)" ], [ 7920, "M11" ], 
  [ 9828, "L2(27)" ], [ 12180, "L2(29)" ], [ 14880, "L2(31)" ], 
  [ 20160, "A8" ,"L3(4)" ], [ 25308, "L2(37)" ], 
  [ 25920, "U4(2)" ], [ 29120, "Sz(8)" ], [ 32736, "L2(32)" ], 
  [ 34440, "L2(41)" ], [ 39732, "L2(43)" ], [ 51888, "L2(47)" ], 
  [ 58800, "L2(49)" ], [ 62400, "U3(4)" ], [ 74412, "L2(53)" ], 
  [ 95040, "M12" ], [ 102660, "L2(59)" ], [ 113460, "L2(61)" ], 
  [ 126000, "U3(5)" ], [ 150348, "L2(67)" ], [ 175560, "J1" ], 
  [ 178920, "L2(71)" ], [ 181440, "A9" ], [ 194472, "L2(73)" ], 
  [ 246480, "L2(79)" ], [ 262080, "L2(64)" ], [ 265680, "L2(81)" ], 
  [ 285852, "L2(83)" ], [ 352440, "L2(89)" ], [ 372000, "L3(5)" ], 
  [ 443520, "M22" ], [ 456288, "L2(97)" ], [ 515100, "L2(101)" ], 
  [ 546312, "L2(103)" ], [ 604800, "J2" ], [ 612468, "L2(107)" ], 
  [ 647460, "L2(109)" ], [ 721392, "L2(113)" ], [ 885720, "L2(121)" ], 
  [ 976500, "L2(125)" ], [ 979200, "Sp4(4)" ] ];

SimpleGroupName := function(ident)
  return Filtered(NNNSimpleGroupNames, x -> x[1] = ident[1])[1][ident[2]+1];
end;

PerfectGroupRecord := function(ident)
  return ShallowCopy(PERFGRP[PerfGrpLoad(ident[1])][ident[2]]);
end;

PerfectGroupName := function(ident)
    return PerfectGroupRecord(ident)[2];
end;

ConvertSimpleIdentToPerfect := function(indent)
   local L, size;
   size := indent[1];
   L := Filtered(PERFRec.sizeNumberSimpleGroup, x -> x[1] = size);
   return L[indent[2]];
end;

CompositionFactorsPerfectGroup := function(ident)
   local factors, f;
   factors := PerfectGroupRecord(ident)[5];
   if IsInt(factors) then factors := [factors]; fi;
   f := function(simpleindex) 
      local perf_ident;
      perf_ident := ShallowCopy(PERFRec.sizeNumberSimpleGroup[simpleindex]);
      if perf_ident[1] = 20160 then 
         perf_ident[2] := perf_ident[2] - 3;
      fi;
      return perf_ident;
    end;
    return List(factors, f);
end;

InvariantsOfAbelianSubgroups := function(G)
   local S, A;
   S := ConjugacyClassesSubgroups(G);
   A := Filtered(S, h -> IsAbelian(Representative(h)));
   return List(A, h -> AbelianInvariants(Representative(h)));
end;
