#include <stdio.h>
#include <assert.h>

/* Computes the rank of a matrix mod the prime < 2^15 */
/* Input file format is:                              */
/*     prime rows cols  data                          */
/*                                                    */
/* Works by reducing matrix to semi-echelon form,     */
/* with -1's on the pivots                            */

void fill_matrix(int rows, int cols, int *A, int p){
  int i, j,tmp;
  int* curr_row;

  curr_row = A;
  for (i = 0; i < rows; i++, curr_row += cols){
    for (j = 0; j < cols; j++){
      scanf("%d",&tmp);
      tmp = tmp % p;
      if (tmp < 0) tmp += p;
      curr_row[j] = tmp;
    }
  }
}

int* table_of_inverses(int p){
  int* inv;
  FILE* table;
  int i;

  inv = (int *)malloc(p*sizeof(int));
  table = fopen("inv31991", "r");
  for (i = 0; i < p; i++)
    fscanf(table, "%d", &inv[i]);
  return inv;
}
	
/* below from SnapPea */
int euclidean_algorithm(
	int	m,
        int	n,
	int	*a,
        int	*b)
{
	/*
	 *	Given two long integers m and n, use the Euclidean algorithm to
	 *	find integers a and b such that a*m + b*n = g.c.d.(m,n).
	 *
	 *	Recall the Euclidean algorithm is to keep subtracting the
	 *	smaller of {m, n} from the larger until one of them reaches
	 *	zero.  At that point the other will equal the g.c.d.
	 *
	 *	As the algorithm progresses, we'll use the coefficients
	 *	mm, mn, nm, and nn to express the current values of m and n
	 *	in terms of the original values:
	 *
	 *		current m = mm*(original m) + mn*(original n)
	 *		current n = nm*(original m) + nn*(original n)
	 */

	 int	mm,
				mn,
				nm,
				nn,
				quotient;

	/*
	 *	Begin with a quick error check.
	 */
	assert(!(m == 0 && n == 0));

	/*
	 *	Initially we have
	 *
	 *		current m = 1 (original m) + 0 (original n)
	 *		current n = 0 (original m) + 1 (original n)
	 */

	mm = nn = 1;
	mn = nm = 0;

	/*
	 *	It will be convenient to work with nonnegative m and n.
	 */

	if (m < 0)
	{
		m = - m;
		mm = -1;
	}

	if (n < 0)
	{
		n = - n;
		nn = -1;
	}

	while (1)
	{
		/*
		 *	If m is zero, then n is the g.c.d. and we're done.
		 */
		if (m == 0)
		{
			*a = nm;
			*b = nn;
			return n;
		}

		/*
		 *	Let n = n % m, and adjust the coefficients nm and nn accordingly.
		 */
		quotient = n / m;
		nm -= quotient * mm;
		nn -= quotient * mn;
		n  -= quotient * m;

		/*
		 *	If n is zero, then m is the g.c.d. and we're done.
		 */
		if (n == 0)
		{
			*a = mm;
			*b = mn;
			return m;
		}

		/*
		 *	Let m = m % n, and adjust the coefficients mm and mn accordingly.
		 */
		quotient = m / n;
		mm -= quotient * nm;
		mn -= quotient * nn;
		m  -= quotient * n;
	}

	/*
	 *	We never reach this point.
	 */
}


int Zq_inverse(
	 int	p,
	 int	q)
{
	 int	a,
				b,
				g;

	/*
	 *	Make sure 0 < p < q.
	 */
	 
	assert(0 < p && p < q );

	/*
	 *	Find a and b such that ap + bq = gcd(p,q) = 1.
	 */

	g = euclidean_algorithm(p, q, &a, &b);

	/*
	 *	Check that p and q are relatively prime.
	 */

	assert(g == 1);

	/*
	 *		ap + bq = 1
	 *	=>	ap = 1 (mod q)
	 *	=>	The inverse of p in Z/q is a.
	 *
	 *	Normalize a to the range (0, q).
	 *
	 *	[My guess is that a must always fall in the range -q < a < q,
	 *	in which case the follwing code would simplify to
	 *
	 *		if (a < 0)
	 *			a += q;
	 *
	 *	but I haven't worked out a proof.]
	 */

	while (a < 0)
		a += q;
	while (a > q)
		a -= q;

	return a;
}


int main(void){
  int p, rows, cols, inv, lead;
  int *A;  /* the array itself */
  int i,j,k;
  int rank = 0;
  int* pivots;  /* pivots[i] is the row with pivot in the ith column */
  int* inverses;
  int* curr_row, *other_row;

  scanf("%d%d%d", &p, &rows, &cols);
  assert(0 < p && p < 32768);
  
  A = (int*)malloc(rows*cols*sizeof(int));
  fill_matrix(rows, cols, A,p);

  pivots = (int*)malloc(cols*sizeof(int));
  for (j = 0; j < cols; j++) 
    pivots[j] = -1;
  
  curr_row = A;
  for (i = 0; i < rows; i++, curr_row += cols){
    for (j = 0;  j < cols; j++){
      if (curr_row[j]){
	if (pivots[j] != -1){
	  other_row = A + cols*pivots[j];
	  lead = curr_row[j];
	  for (k = j; k < cols; k++)
	    curr_row[k] = (curr_row[k] + lead * other_row[k]) % p;
	}
	else{
	  pivots[j] = i;
	  rank++;
	  inv = Zq_inverse(curr_row[j], p);
	  for (k = j; k < cols; k++)
	    curr_row[k] = (-inv*curr_row[k]) %p + p;   /* make pivot -1 */
	  break;
	}
      }
    }
    if (rank == cols ) break;  /* max rank already, quit */
  }

  printf("RANK: %d\n", rank);
  
  /* DIAGNOSITIC INFORMATION:
  for (i = 0; i < rows; i++, curr_row += cols){
    for (j = 0;  j < cols; j++){
      printf("%d ", A[i*cols + j]);
    }
    printf("\n");
  }
  for (j = 0; j < cols; j++){
    printf("%d ", pivots[j]);
  }
  */

  free(A);
  free(pivots);
  /* free(inverses);*/
}
	      
  



